"use strict";
this.name        = "market_inquirer"; 
this.author      = "spara"; 
this.copyright   = "2013 - 2014 Mika Spara";
this.licence     = "CC BY-NC-SA 3.0"; 
this.description = "Show market information from main and oxp stations"; 
this.version     = "1.9";

//1.9 mission screen replaced with mfd and market hack is removed
//1.8 brought mission screens back and tidied up a bit
//1.7 removed in-flight mission screens and equipment and replaced them with a market hack.
//1.6 rename Navigation buoy to main station.
//1.6 hide the beacon of the station you're docked
//1.5 added pager to market viewer
//1.5 added sirf to inquirer stations
//1.5 limit station service to inquirerStations
//1.5 distances interface
//1.4 added more stations and inquirer_station role to the inquirer.
//1.4 moved station definitions to startUp.

this.startUp = function() {
	this.$removeInterfaces();
	//stations noted by market inquirer. any role of the station will do.
	this.$inquirerStations = ["inquirer_station", "sfep_station", "wildShips_kiota", "sothis", "constore", "pagroove_superhub", "casinoship", "rescue_station", "GRS-Station", "SIRF-YARD"];
	
	//special equipment
	if (player.ship.equipmentStatus("EQ_ADVANCED_COMPASS") === "EQUIPMENT_OK")
		player.ship.awardEquipment("EQ_MARKET_INQUIRER");
	else player.ship.removeEquipment("EQ_MARKET_INQUIRER");

	this.$commodities = ["food","textiles","radioactives","slaves","liquor_wines","luxuries","narcotics","computers","machinery","alloys","firearms","furs","minerals","gold","platinum","gem_stones","alien_items"];
	this.$commodity = 0;
	
};

this.$changeCommodity = function(action) {
	this.$commodity = this.$commodity + action;
	if (this.$commodity < 0)
		this.$commodity = this.$commodities.length - 1;
	else if (this.$commodity > this.$commodities.length - 1)
		this.$commodity = 0;
	this.$commodityPrices();
}

this.shipLaunchedFromStation = this.shipExitedWitchspace = function() {
	this.$commodityPrices();
}

this.$commodityPrices = function() {
	if (player.ship.equipmentStatus("EQ_MARKET_INQUIRER") !== "EQUIPMENT_OK") {
		player.ship.setMultiFunctionText("inquirer_mfd", null);
		return;
	}
	if (system.isInterstellarSpace || !system.sun || system.sun.isGoingNova || system.sun.hasGoneNova || !system.mainStation)
		var message = "No market data available.";
	else {
		var hairSpace = String.fromCharCode(31);
		var tab = "";
		for (var i = 0; i < 10; i++)
			tab = tab + hairSpace;
		var noUnits = tab + "   --";
		var message = "Commodity: " + system.mainStation.market[this.$commodities[this.$commodity]].displayName + "\n";
		var stations = system.filteredEntities(this, this.$stations, player.ship);
		if (stations.length < 8) message = message + "\n";
		var price = formatCredits(system.mainStation.market[this.$commodities[this.$commodity]].price);
		var quantity = system.mainStation.market[this.$commodities[this.$commodity]].quantity;
		if (quantity === 0) quantity = noUnits;
		else quantity = formatUnits(quantity);
		message = message + price + " /" + quantity + " Main Station\n";
		for (var i = 0; i < stations.length; i++) {
			var name = stations[i].displayName;
			price = formatCredits(stations[i].market[this.$commodities[this.$commodity]].price);
			quantity = stations[i].market[this.$commodities[this.$commodity]].quantity;
			if (quantity === 0) quantity = noUnits;
			else quantity = formatUnits(quantity);
			message = message + truncate(price + " /" + quantity + " " + name) + "\n";
			
		}
	}
	player.ship.setMultiFunctionText("inquirer_mfd", message);
	//helper function to truncate lines
	function truncate(line) {
		var lastChar = "";
		while (global.defaultFont.measureString(line) > 14) {
			lastChar = line.charAt(line.length - 1);
			line = line.slice(0, -1);
		}
		if (lastChar === "") return line;
		if (line.charAt(line.length - 1) === " " || lastChar === " ")
			return line;
		if (line.charAt(line.length - 2) === " ") 
			return line.slice(0, -1);
		line = line + ".";
		while (global.defaultFont.measureString(line) > 14) {
			line = line.slice(0, -2);
			line = line + ".";
		}
		return line;
	}
	//helper function to format credits
	function formatCredits(credits) {
		if (credits < 99.5) 
			return "    " + global.formatCredits(credits/10, true, false);
		else if (credits < 999.5)
			return "  " + global.formatCredits(credits/10, true, false);
		else return global.formatCredits(credits/10, true, false);
	};
	//helper function to format units
	function formatUnits(units) {
		if (units < 10) return "    " + units;
		else if (units < 100 ) return "  " + units;
		else return units;
	};
}

//Add f4 interfaces when docked
this.shipDockedWithStation = function(station) {
	if (player.ship.equipmentStatus("EQ_ADVANCED_COMPASS") === "EQUIPMENT_OK" || this.$stations(station) || station.isMainStation)
		this.$marketInquirerInterface();
};

//remove interfaces
this.$removeInterfaces = this.shipWillLaunchFromStation = function() {
	player.ship.dockedStation.setInterface("inquirer_market",null);
	player.ship.dockedStation.setInterface("inquirer_distances",null);
}

//Add interfaces
this.$marketInquirerInterface = function() {
	player.ship.dockedStation.setInterface("inquirer_market",{
		title: "Commodity markets",
		category: "Commerce",
		summary: "Show commodity markets of selected stations in system.",
		callback: this.$showMarkets.bind(this)
	});
	player.ship.dockedStation.setInterface("inquirer_distances",{
		title: "In-system distances",
		category: "Informational",
		summary: "Show distances to the star, planets, moons and beacons in system.",
		callback: this.$showDistances.bind(this)
	});	
};

//function to filter suitable stations
this.$stations = function(entity) {	
	if (entity.isStation) {
		var k;
		for (k = 0; k < this.$inquirerStations.length; k++) {
			if (entity.hasRole(this.$inquirerStations[k]))
				return true;
		}	
	}
	return false;
};

//build and show markets screen
this.$showMarkets = function(page) {
	
	var text = "";
	var hairSpace = String.fromCharCode(31);
	var stations = new Array();
	//No asc
	if (player.ship.docked && (player.ship.equipmentStatus("EQ_ADVANCED_COMPASS") !== "EQUIPMENT_OK" && !player.ship.dockedStation.isMainStation && !this.$stations(player.ship.dockedStation))) {
		text = "No market data available.";
		this.$removeInterfaces();
	}
	//sanity check
	else if (system.isInterstellarSpace || !system.sun || system.sun.isGoingNova || system.sun.hasGoneNova || !system.mainStation) {
		text = "No market data available.";
	}
	//let's rock
	else {
		if (typeof page === 'undefined' || page === 'inquirer_market') page = 0;
		var commodities  = ["food","textiles","radioactives","slaves","liquor_wines","luxuries","narcotics","computers","machinery","alloys","firearms","furs","minerals","gold","platinum","gem_stones","alien_items"];	
		var i,j, quantity;
		//description column width
		var descriptionWidth = Math.ceil(global.defaultFont.measureString("Liquor/Wines:"));
		//width for market data columns
		var dataWidth = 31 - descriptionWidth;
		//minimum width for market data column
		var minColWidth = global.defaultFont.measureString(" "+formatCredits(1020)+" / 64 ");
		//maximum number of data columns per page
		var maxCols = Math.floor(dataWidth / minColWidth);
		//filtered stations
		stations = system.filteredEntities(this, this.$stations, player.ship);
		stations.unshift(system.mainStation);
		//handle swapping back to start
		if (page > stations.length / maxCols) page = 0;
		var sfepStations = new Array();
		//page handling
		var startInd = page * maxCols;
		//show full pages
		if (startInd > 0 && stations.length > maxCols && startInd + maxCols > stations.length - 1)
			startInd = stations.length - maxCols;
		//select stations to show
		for (i = startInd; i < startInd + maxCols; i++) {
			sfepStations.push(stations[i]);
			if (i > stations.length - 2) break;
		}
		//calculate actual data column width
		if (sfepStations.length >= maxCols) var colWidth = dataWidth / maxCols;
		else var colWidth = (dataWidth) / (sfepStations.length);
		//array for formatted (truncated and centered) stations names
		var statNames = new Array();
		//add the station names to the array
		for (i = 0; i < sfepStations.length; i++) {
			if (sfepStations[i].isMainStation)
				statNames.push(format("Main Station", colWidth));
			else statNames.push(format(sfepStations[i].displayName, colWidth));
		}
		//tabulate over the description column
		while (global.defaultFont.measureString(text) < descriptionWidth - 0.1)
			text = text + " ";
		while (global.defaultFont.measureString(text) < descriptionWidth)
			text = text + hairSpace;

		//add and tabulate station names
		for (i = 0; i < statNames.length; i++) {
			text = text + statNames[i];
			while (global.defaultFont.measureString(text) < (descriptionWidth) + colWidth * (i + 1) - 0.1)
				text = text + " ";
			while (global.defaultFont.measureString(text) < (descriptionWidth) + colWidth * (i + 1))
				text = text + hairSpace;
			if (i === maxCols - 1) break;
		}
		text = text + "\n\n";
		//then the prices
		for (j = 0; j < commodities.length; j++) {
			var line = "";
			//add commodity name
			line = system.mainStation.market[commodities[j]].displayName+": ";
			//tabulate
			while (global.defaultFont.measureString(line) < descriptionWidth - 0.1)
				line = line + " ";
			while (global.defaultFont.measureString(line) < descriptionWidth)
				line = line + hairSpace;
			//add prices/quantities
			for (i = 0; i < sfepStations.length; i++) {
				quantity = sfepStations[i].market[commodities[j]].quantity;
				if (quantity === 0) quantity = " --";
				else quantity = formatUnits(quantity);
				line = line + format(formatCredits(sfepStations[i].market[commodities[j]].price)+" / "+quantity, colWidth);
				while (global.defaultFont.measureString(line) < descriptionWidth + colWidth * (i + 1) - 0.1)
					line = line + " ";
				while (global.defaultFont.measureString(line) < descriptionWidth + colWidth * (i + 1))
					line = line + hairSpace;
				if (i === maxCols - 1) break;
			}
			text = text + line + "\n";
		}
	}
	//select the correct background image
	var bgImage = "sfep_flight.png";
	if (player.ship.docked) bgImage = "sfep_docked.png";
	//select correct options
	var options = {
		"1_NEXT" : "Swap",
		"2_QUIT" : "Exit",
	}
	if (stations.length <= maxCols || stations.length === 0) {
		options = {
			"1_QUIT" : "Exit"
		}
	}
	//finally, the mission screen
	mission.runScreen({
		title: "Commodity Markets of " + system.name,
		message: text,
		choices: options,
		background: bgImage
	},
	function (choice) {
		if (choice === "1_NEXT") worldScripts.market_inquirer.$showMarkets(page + 1);
	});
	//helper function to format credits
	function formatCredits(credits) {
		if (credits < 99.5) 
			return "    " + global.formatCredits(credits/10, true, false);
		else if (credits < 999.5)
			return "  " + global.formatCredits(credits/10, true, false);
		else return global.formatCredits(credits/10, true, false);
	};
	//helper function to format units
	function formatUnits(units) {
		if (units < 10) return "  " + units;
		else return units;
	};
	//helper function to truncate and center station names
	function format(string, width) {
		//if too wide, truncate
		if (global.defaultFont.measureString(string) >= width) {
			var lastChar = "";
			while (global.defaultFont.measureString(string) > width - 1) {
				lastChar = string.charAt(string.length - 1);
				string = string.slice(0, -1);
			}
			if (string.charAt(string.length - 1) === " ")
				string = string.slice(0, -1);
			else if (lastChar !== " ")
				string = string + ".";
		}
		var stringWidth = global.defaultFont.measureString(string);
		//if too narrow, center 
		if (stringWidth < width) {
			var fill = (width - stringWidth) / 2;
			var tab = " ";
			while (global.defaultFont.measureString(tab) < fill)
				tab = tab + " ";
			string = tab + string;
		};
		return string;
	};
};

//build and show distances screen
this.$showDistances = function() {
	var text = "";
	//No asc
	if (player.ship.docked && player.ship.equipmentStatus("EQ_ADVANCED_COMPASS") !== "EQUIPMENT_OK" && !player.ship.dockedStation.isMainStation && !this.$stations(player.ship.dockedStation)) {
		text = "No distance data available.";
		this.$removeInterfaces();
	}
	//sanity check;
	else if (system.isInterstellarSpace) {
		text = "No distance data available.";
	}
	else {
		//define distance unit
		var unitBase = 1000;
		var unit = "km";
		//use the same unit as Updating TSC
		if (worldScripts.Updating_TSC_world_script) {
			unitBase = worldScripts.Updating_TSC_world_script.$ostronomicalUnit;
			unit = worldScripts.Updating_TSC_world_script.$distUnit;
		}
		var i;
		var lines = new Array();
		//star
		if (system.sun) {
			var name = "Sun";
			if (system.info.sun_name) name = system.info.sun_name;
			lines.push(["Star:",""]);
			lines.push(["* " +name, dist(system.sun)]);
			lines.push([" ",""]);
		};
		//planets
		var planets = system.filteredEntities(this, planets, player.ship);
		if (planets.length !== 0) {
			lines.push(["Planets:",""]);
			for (i = 0; i < planets.length; i++) {
				if (planets[i].displayName)
					lines.push(["* " + planets[i].displayName.replace(" (Planet)",""),dist(planets[i])]);
				else lines.push(["* Planet " + (i+1),dist(planets[i])]);
			}
			lines.push([" ",""]);
		};
		//moons
		var moons = system.filteredEntities(this, moons, player.ship);
		if (moons.length !== 0) {
			lines.push(["Moons:",""]);
			for (i = 0; i < moons.length; i++) {
				if (moons[i].displayName)
					lines.push(["* " + moons[i].displayName.replace(" (Moon)",""),dist(moons[i])]);
				else lines.push(["* Moon " + (i+1),dist(moons[i])]);
			}
			lines.push([" ",""]);
		};
		//beacons
		var beacons = system.filteredEntities(this, beacons, player.ship);
		if (beacons.length !== 0) {
			lines.push(["Beacons:",""]);
			for (i = 0; i < beacons.length; i++) {
				if (beacons[i].displayName) {
					if (!player.ship.docked || player.ship.position.distanceTo(beacons[i]) > 10E4) {
						if (beacons[i].displayName === "Navigation Buoy" && system.mainStation.position.distanceTo(beacons[i]) <= 10E4)
							lines.push(["* " + "Main Station Buoy", dist(beacons[i])]);
						else
							lines.push(["* " + beacons[i].displayName, dist(beacons[i])]);
					}
				}
				else lines.push(["* Beacon " + (i+1)+": ", dist(beacons[i])]);
			}
		}
		//build lines for mission screen
		var column1 = lines;
		var column2 = new Array();
		if (column1.length > 19)
			column2 = column1.splice(19, column1.length - 19);
		for (i = 0; i < column1.length; i++) {
			if (column2.length > i)
				text = text + formatStrToCol(column1[i][0], column1[i][1], 15.5) + formatStrToCol(column2[i][0], column2[i][1], 15.5) + "\n";
			else
			text = text + formatStrToCol(column1[i][0], column1[i][1], 31) + "\n";
		}
	}
	//show mission screen
	var bgImage = "sfep_flight.png";
	if (player.ship.docked) bgImage = "sfep_docked.png";
	var options = {
			"1_QUIT" : "Exit"
	}
	mission.runScreen({
		title: "Distances of " + system.name,
		message: text,
		choices: options,
		background: bgImage
	});

	//helper functions
	function planets(entity) {return (entity.isPlanet && entity.hasAtmosphere)};
	function moons(entity) {return (entity.isPlanet && !entity.hasAtmosphere)};
	function beacons(entity) {return entity.isBeacon};
	function dist(entity) {
		var distInKm = (player.ship.position.distanceTo(entity) - entity.collisionRadius)/unitBase;
		return (distInKm.toFixed(3) + " " + unit);
	};
	//format columns
	function formatStrToCol(name, distStr, colWidth) {
		//if too wide, truncate
		var punct = "";
		if (distStr !== "") punct = ": ";
		if (global.defaultFont.measureString(name+punct+distStr) > colWidth) {
			var lastChar = "";
			while (global.defaultFont.measureString(name+punct+distStr) > colWidth) {
				lastChar = name.charAt(name.length - 1);
				name = name.slice(0, -1);
			}
			if (name.charAt(name.length - 1) === " ")
				name = name.slice(0, -1);
			else if (lastChar !== " ")
				name = name + ".";
		}
		//if too narrow, add spaces to end 
		while (global.defaultFont.measureString(name+punct+distStr) < colWidth) {
			distStr = distStr + " ";
		};
		if (distStr.charAt(distStr.length - 1) === " ")
			distStr = distStr.slice(0, -1);
		return name + punct + distStr;
	};	
}

//Bind inquirer to the asc, does not catch awarding asc by script though.
this.playerBoughtEquipment = function(equipmentKey) {
	if (equipmentKey === "EQ_ADVANCED_COMPASS")
		player.ship.awardEquipment("EQ_MARKET_INQUIRER");
	if (equipmentKey === "EQ_ADVANCED_COMPASS_REMOVAL")
		player.ship.removeEquipment("EQ_MARKET_INQUIRER");		
}

this.playerBoughtNewShip = function(ship) {
	if (ship.equipmentStatus("EQ_ADVANCED_COMPASS") === "EQUIPMENT_OK") 
		player.ship.awardEquipment("EQ_MARKET_INQUIRER");
	else
		player.ship.removeEquipment("EQ_MARKET_INQUIRER");		
}

this.equipmentDamaged = this.equipmentDestroyed = function(equipment) {
	if (equipment === "EQ_ADVANCED_COMPASS")
		player.ship.removeEquipment("EQ_MARKET_INQUIRER");
}

this.equipmentRepaired = function(equipment) {
	if (equipment === "EQ_ADVANCED_COMPASS")
		player.ship.awardEquipment("EQ_MARKET_INQUIRER");
}
